@@ -22,11 +22,13 @@ unsafe impl Allocator for Sensitive {
 			return Err(AllocError);
 		}
 
-		let alloc = GuardedAlloc::<{ Sensitive::GUARD_PAGES }>::new(layout.size(), Protection::ReadWrite).map_err(|_| AllocError)?;
+		let alloc = GuardedAlloc::<{ Self::GUARD_PAGES }>::new(layout.size(), Protection::ReadWrite).map_err(|_| AllocError)?;
 
 		if likely(!alloc.inner().is_empty()) {
 			// Attempt to lock memory
-			let _ = alloc.inner().lock();
+			#[allow(unused_must_use)] {
+				alloc.inner().lock();
+			}
 		}
 
 		Ok(alloc.into_slice())
@@ -212,6 +214,9 @@ mod tests {
 		// Allocation should not move
 		assert_eq!(alloc_0.cast::<u8>(), alloc_1.cast::<u8>());
 
+		// Ensure TLB flush
+		std::thread::yield_now();
+
 		for i in 0 .. size_1 {
 			assert_eq!(unsafe { bp.load(ptr.add(i)) }, Ok(0));
 		}
