@@ -1,14 +1,10 @@
 #![feature(
 	allocator_api,
-	available_parallelism,
-	cell_update,
 	core_intrinsics,
-	maybe_uninit_extra,
 	maybe_uninit_slice,
 	nonnull_slice_from_raw_parts,
 	slice_ptr_get,
 	slice_ptr_len,
-	vec_spare_capacity,
 )]
 
 pub mod auxiliary;
