@@ -4,7 +4,7 @@ use crate::auxiliary::align;
 use crate::traits::{AsPages, Protectable};
 
 use std::convert::TryInto;
-use std::intrinsics::{likely, unlikely};
+use std::intrinsics::likely;
 use std::io::Error;
 use std::marker::PhantomData;
 use std::mem::{MaybeUninit, ManuallyDrop};
@@ -51,14 +51,17 @@ pub enum Protection {
 }
 
 /// Memory pages
+#[must_use]
 #[derive(Debug)]
 pub struct Pages<'t>(NonNull<[u8]>, PhantomData<&'t ()>);
 
 /// Memory page allocation
+#[must_use]
 #[derive(Debug)]
 pub struct Allocation(NonNull<[u8]>);
 
 /// Guarded memory page allocation
+#[must_use]
 #[derive(Debug)]
 pub struct GuardedAlloc<const N: usize = 1>(Allocation);
 
@@ -93,12 +96,14 @@ fn init() {
 }
 
 impl<'t> Pages<'t> {
+	#[must_use]
 	pub fn granularity() -> usize {
 		init();
 
 		unsafe { PAGE_SIZE.assume_init() }
 	}
 
+	#[must_use]
 	pub fn align(offset: usize) -> usize {
 		align(offset, Self::granularity())
 	}
@@ -132,34 +137,41 @@ impl<'t> Pages<'t> {
 	///
 	/// `ptr` must be non‐null and aligned to a multiple of the [page size](Self::granularity) and `size` must be a
 	/// multiple of the [page size](Self::granularity).
+	///
+	/// # Panics
+	///
+	/// May panic if `ptr` is null or if `ptr` or `size` are not properly aligned.
 	pub unsafe fn from_ptr<T>(ptr: *mut T, size: usize) -> Pages<'t> {
 		Self::from_raw_parts(NonNull::new(ptr.cast::<u8>()).unwrap(), size)
 	}
 
+	#[must_use]
 	pub fn as_ptr<T>(&self) -> *mut T {
 		debug_assert!(std::mem::align_of::<T>() < Self::granularity());
 		self.0.as_ptr().cast::<T>()
 	}
 
-	#[inline]
-	pub fn into_slice(self) -> NonNull<[u8]> {
+	#[must_use] #[inline]
+	pub const fn into_slice(self) -> NonNull<[u8]> {
 		self.0
 	}
 
-	#[inline]
+	#[must_use] #[inline]
 	pub fn size(&self) -> usize {
 		self.0.len()
 	}
 
+	#[must_use]
 	pub fn len(&self) -> usize {
 		self.size() / Self::granularity()
 	}
 
-	#[inline]
+	#[must_use] #[inline]
 	pub fn is_empty(&self) -> bool {
 		self.size() == 0
 	}
 
+	#[allow(clippy::missing_errors_doc)]
 	pub fn protect(&self, prot: Protection) -> Result<(), Error> {
 		#[cfg(unix)] {
 			use libc::mprotect;
@@ -183,6 +195,7 @@ impl<'t> Pages<'t> {
 		}
 	}
 
+	#[allow(clippy::missing_errors_doc)]
 	pub fn lock(&self) -> Result<(), Error> {
 		#[cfg(unix)] {
 			use libc::mlock;
@@ -203,6 +216,7 @@ impl<'t> Pages<'t> {
 		}
 	}
 
+	#[allow(clippy::missing_errors_doc)]
 	pub fn unlock(&self) -> Result<(), Error> {
 		#[cfg(unix)] {
 			use libc::munlock;
@@ -223,6 +237,7 @@ impl<'t> Pages<'t> {
 		}
 	}
 
+	#[must_use]
 	pub fn pages(&'t self, range: Range<usize>) -> Option<Pages<'t>> {
 		if likely(range.start < self.len() && range.end <= self.len()) {
 			Some(unsafe {
@@ -236,6 +251,7 @@ impl<'t> Pages<'t> {
 }
 
 impl Allocation {
+	#[must_use]
 	pub fn granularity() -> usize {
 		#[cfg(unix)] {
 			Pages::granularity()
@@ -248,6 +264,7 @@ impl Allocation {
 		}
 	}
 
+	#[must_use]
 	pub fn align(offset: usize) -> usize {
 		align(offset, Self::granularity())
 	}
@@ -258,7 +275,7 @@ impl Allocation {
 	///
 	/// `slice` must have been previously generated by a call to [`into_slice`](Self::into_slice) and must not be
 	/// aliased by any other [`Allocation`].
-	pub unsafe fn from_slice(slice: NonNull<[u8]>) -> Allocation {
+	pub unsafe fn from_slice(slice: NonNull<[u8]>) -> Self {
 		// Assert correct alignment
 		debug_assert_eq!(slice.as_ptr().cast::<u8>().align_offset(Self::granularity()), 0);
 		debug_assert_eq!(Self::align(slice.len()), slice.len());
@@ -272,7 +289,7 @@ impl Allocation {
 	///
 	/// `ptr` must have been previously generated by a call to [`into_ptr`](Self::into_ptr) and must not be aliased
 	/// by any other [`Allocation`]. `size` must match the size of the original `Allocation`.
-	pub unsafe fn from_raw_parts(ptr: NonNull<u8>, size: usize) -> Allocation {
+	pub unsafe fn from_raw_parts(ptr: NonNull<u8>, size: usize) -> Self {
 		Self::from_slice(NonNull::slice_from_raw_parts(ptr, size))
 	}
 
@@ -282,40 +299,47 @@ impl Allocation {
 	///
 	/// `ptr` must have been previously generated by a call to [`into_ptr`](Self::into_ptr) and must not be aliased
 	/// by any other [`Allocation`]. `size` must match the size of the original `Allocation`.
-	pub unsafe fn from_ptr<T>(ptr: *mut T, size: usize) -> Allocation {
+	///
+	/// # Panics
+	///
+	/// May panic if `ptr` is null or `ptr` or `size` are not properly aligned.
+	pub unsafe fn from_ptr<T>(ptr: *mut T, size: usize) -> Self {
 		Self::from_raw_parts(NonNull::new(ptr.cast::<u8>()).unwrap(), size)
 	}
 
+	#[must_use]
 	pub fn as_ptr<T>(&self) -> *mut T {
 		debug_assert!(std::mem::align_of::<T>() < Self::granularity());
 		self.0.as_ptr().cast::<T>()
 	}
 
-	#[inline]
+	#[must_use] #[inline]
 	pub fn into_ptr<T>(self) -> *mut T {
 		ManuallyDrop::new(self).as_ptr()
 	}
 
-	#[inline]
+	#[must_use] #[inline]
 	pub fn into_slice(self) -> NonNull<[u8]> {
 		ManuallyDrop::new(self).0
 	}
 
-	#[inline]
+	#[must_use] #[inline]
 	pub fn size(&self) -> usize {
 		self.0.len()
 	}
 
+	#[must_use]
 	pub fn len(&self) -> usize {
 		self.size() / Pages::granularity()
 	}
 
-	#[inline]
+	#[must_use] #[inline]
 	pub fn is_empty(&self) -> bool {
 		self.size() == 0
 	}
 
-	pub fn new(size: usize, prot: Protection) -> Result<Allocation, Error> {
+	#[allow(clippy::missing_errors_doc)]
+	pub fn new(size: usize, prot: Protection) -> Result<Self, Error> {
 		let size = Self::align(size);
 
 		#[cfg(unix)] {
@@ -341,7 +365,11 @@ impl Allocation {
 		}
 	}
 
-	pub fn shrink(self, size: usize) -> Result<Allocation, Error> {
+	/// # Panics
+	///
+	/// May panic if `size` is not smaller than the current size.
+	#[allow(clippy::missing_errors_doc)]
+	pub fn shrink(self, size: usize) -> Result<Self, Error> {
 		assert!(size < self.0.len());
 
 		let size = Pages::align(size);
@@ -371,6 +399,7 @@ impl Allocation {
 		}
 	}
 
+	#[must_use]
 	pub fn pages(&self, range: Range<usize>) -> Option<Pages> {
 		if likely(range.start < self.len() && range.end <= self.len()) {
 			Some(unsafe {
@@ -388,18 +417,16 @@ impl Drop for Allocation {
 		#[cfg(unix)] {
 			use libc::munmap;
 
-			if unlikely(unsafe { munmap(self.as_ptr::<c_void>(), self.0.len()) } != 0) {
-				panic!("{}", Error::last_os_error());
-			}
+			assert_eq!(unsafe { munmap(self.as_ptr::<c_void>(), self.0.len()) }, 0,
+			"{}", Error::last_os_error());
 		}
 
 		#[cfg(windows)] {
 			use winapi::um::memoryapi::VirtualFree;
 			use winapi::um::winnt::MEM_RELEASE;
 
-			if unlikely(unsafe { VirtualFree(self.as_ptr::<c_void>(), 0, MEM_RELEASE) } == 0) {
-				panic!("{}", Error::last_os_error());
-			}
+			assert_ne!(unsafe { VirtualFree(self.as_ptr::<c_void>(), 0, MEM_RELEASE) }, 0,
+			"{}", Error::last_os_error());
 		}
 	}
 }
@@ -407,18 +434,22 @@ impl Drop for Allocation {
 impl<const N: usize> GuardedAlloc<N> {
 	pub const GUARD_PAGES: usize = N;
 
+	#[must_use]
 	pub fn guard_size() -> usize {
 		Self::GUARD_PAGES * Pages::granularity()
 	}
 
+	#[must_use]
 	pub fn outer_size(size: usize) -> usize {
 		Allocation::align(size + 2 * Self::guard_size())
 	}
 
+	#[must_use]
 	pub fn inner_size(size: usize) -> usize {
 		Self::outer_size(size) - 2 * Self::guard_size()
 	}
 
+	#[allow(clippy::missing_errors_doc)]
 	pub fn new(size: usize, prot: Protection) -> Result<Self, Error> {
 		let alloc = Self(Allocation::new(Self::outer_size(size), Protection::NoAccess)?);
 
@@ -429,6 +460,7 @@ impl<const N: usize> GuardedAlloc<N> {
 		Ok(alloc)
 	}
 
+	#[allow(clippy::missing_panics_doc)]
 	pub fn inner(&self) -> Pages {
 		self.0.pages(Self::GUARD_PAGES .. self.0.len() - Self::GUARD_PAGES).unwrap()
 	}
@@ -457,10 +489,15 @@ impl<const N: usize> GuardedAlloc<N> {
 	///
 	/// `ptr` must have been previously generated by a call to [`into_slice`](Self::into_slice) and must not be
 	/// aliased by any other [`GuardedAlloc`]. `size` must match the size of the original [`GuardedAlloc`].
+	///
+	/// # Panics
+	///
+	/// May panic if `base` is null or if `base` or `inner` are not properly aligned.
 	pub unsafe fn from_ptr<T>(base: *mut T, inner: usize) -> Self {
 		Self::from_raw_parts(NonNull::new(base.cast::<u8>()).unwrap(), inner)
 	}
 
+	#[must_use] #[allow(clippy::missing_panics_doc)]
 	pub fn into_slice(self) -> NonNull<[u8]> {
 		let len = self.0.len();
 		ManuallyDrop::new(self.0).pages(Self::GUARD_PAGES .. len - Self::GUARD_PAGES).unwrap().into_slice()
@@ -470,6 +507,7 @@ impl<const N: usize> GuardedAlloc<N> {
 		unsafe { Pages::from_slice(self.into_slice()) }
 	}
 
+	#[allow(clippy::missing_errors_doc, clippy::missing_panics_doc)]
 	pub fn shrink(self, size: usize) -> Result<Self, Error> {
 		let outer = Self::outer_size(size);
 
@@ -639,6 +677,9 @@ mod tests {
 		let alloc_1 = alloc_0.shrink(size_1).unwrap();
 		assert_eq!(alloc_1.size(), size_1);
 
+		// Ensure TLB flush
+		std::thread::yield_now();
+
 		for i in 0..size_1 {
 			assert_eq!(unsafe { bp.load(ptr.add(i)) }, Ok(0x55));
 		}
@@ -757,6 +798,9 @@ mod tests {
 		// Allocation should not move
 		assert_eq!(alloc_1.inner().as_ptr::<u8>(), ptr);
 
+		// Ensure TLB flush
+		std::thread::yield_now();
+
 		for i in 0 .. size_1 {
 			assert_eq!(unsafe { bp.load(ptr.add(i)) }, Ok(0));
 		}
